################################################################################
## Script for producing figures in Bilbao et al. (2025) The sensitivity of 
## EC-Earth3 decadal predictions to the choice of volcanic forcing dataset: 
## Insights for the next major eruption.
## Script purpose: Compute latlon climatology for dcppA-hindcast for computing 
## anomalies for 2D variables.
## Date: 03/2025
## Author: Roberto Bilbao (roberto.bilbao@bsc.es)
################################################################################

# Load libraries
library(s2dv)
library(startR)

# GCM
gcm <- 'EC-Earth3'

# Variable
vari <- 'tas'

# Exp
dcpp <- 'dcpp*-*cast'

# Table
mod <- 'Amon'

# Startdates
yr1 = '1960'
yr2 = '2015'
sdates = paste(seq(yr1,yr2))

# Forecast time
t1 <- 3
t2 <- 122

#Members
membs <- c('r1i1p1f1','r2i1p1f1','r3i1p1f1','r4i1p1f1','r5i1p1f1','r6i1p1f1','r7i1p1f1','r8i1p1f1','r9i1p1f1','r10i1p1f1')

# Grid
grid <- 'gr'

# Lat and lons
lats <- c(-90, 90)
lons <- c(0, 359.9)

# Path to data
pathdat = ''

#///////////////////////////////////////////
#///////////    DATA LOADING     ///////////
#///////////////////////////////////////////

#--------------------------
# Step1: Load CMIP6 DCPP
#--------------------------

# original_files
repos <- paste0(pathdat,'/$var$_', mod,'_',gcm,'_', dcpp, '_s$sdate$-$memb$_', grid,'_$chunk$.nc')
data <- Start(dat = repos,
              var = vari,
              memb = membs,
              sdate = sdates,
              chunk = chk,
              chunk_depends = 'sdate',
              time = c(t1:t2),
              time_across = 'chunk',
              merge_across_dims = TRUE,
              latitude = values(list(lats[1], lats[2])),
              longitude = values(list(lons[1], lons[2])),
              latitude_reorder = Sort(),
              longitude_reorder = CircularSort(0, 360),
              synonims = list(longitude = c('lon', 'longitude'),
                              latitude = c('lat', 'latitude')),
              return_vars = list(time = c('sdate', 'chunk'),
                                 latitude = NULL, longitude = NULL),
              path_glob_permissive = 7,
              retrieve = F)

#--------------------------
# Step3: Build workflow
#--------------------------

fun_clim <- function(data) {       
         
         #Data
         dcpp = drop(data)
         clim_dcpp <- Clim(dcpp, dcpp, time_dim = "sdate", dat_dim = c("memb"), method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
         clim1_dcpp = clim_dcpp$clim_exp

         # Clim
         dims = dim(dcpp)
         yr_ini = 1960
         yr_fin = 2015
         yr_clim1 = 1971
         yr_clim2 = 2005
         sdats = yr_fin-yr_ini+1
         fmons = dims[3]
         memb = dims[1]

         # Masking period for each forecast time
         time = array(c(c(1:sdats),c(1:fmons)), dim=c(sdats,fmons))*0.0

         for(i in c(1:(yr_fin-yr_ini+1))){
               time[i,] = rep(c(i:(i+9)),each=12)[1:fmons] + yr_ini
               }

         ind = (time < yr_clim1) | (time > yr_clim2)
         ind2 <- InsertDim(ind,1,memb,name = 'memb')

         data2 = dcpp
         data2[ind2] = NA
         dim(data2)

         # New clim
         clim_dcpp2 <- Clim(data2, data2, time_dim = "sdate", dat_dim = c("memb"),
                   method = "clim", ftime_dim = "time", memb_dim = "memb", memb = FALSE, na.rm = TRUE)
         clim2_dcpp = clim_dcpp2$clim_exp

         return(list(clim1_dcpp, clim2_dcpp))

}

step <- Step(fun = fun_clim,
             use_libraries = c('s2dv', 'ClimProjDiags'),
             target_dims = list(data = c('memb', 'sdate', 'time')),
             output_dims = list(clim1_dcpp = c('time'), clim2_dcpp = c('time')))

wf <- AddStep(inputs = list(data),
              step_fun = step)

#-------------------------- 
# Step4: Compute 
#--------------------------

# Calculate chunks 
dim_data <- attr(data, 'Dimensions')
total_data_size <- prod(dim_data) * 8 / 1024 / 1024 / 1024 #Gb 
chunk_latlon <- list(latitude = ceiling(sqrt(total_data_size)/2), longitude = ceiling(sqrt(total_data_size)/2)) 
max_job_num <- ifelse(prod(unlist(chunk_latlon)) < 20, prod(unlist(chunk_latlon)), 20)

# Machine ...
#-------------------user-defined---------------------
queue_host <- ' '
temp_dir <- ' '
ecflow_suite_dir <- ' '
#----------------------------------------------------

output <- Compute(wf[[1]],
                    chunks = chunk_latlon,
                    threads_load = 2,
                    threads_compute = 16,
                    cluster = list(
                      queue_host = queue_host,
                      queue_type = 'slurm',
                      temp_dir = temp_dir,
                      r_module = 'R/4.1.2-foss-2019b',
                      cores_per_job = 16,
                      job_wallclock = '6:00:00',
                      max_jobs = max_job_num,
                      bidirectional = FALSE,
                      polling_period = 10
                    ),
                    ecflow_suite_dir = ecflow_suite_dir,
                    wait = FALSE)

saveRDS(output, file = paste0(gcm,'_',vari,'_dcppclim_collect.Rds'))